from bs4 import BeautifulSoup as bs  # noqa: F401  # install beautifulsoup4
import libcst as cst  # noqa: F401
from sklearn.datasets import load_iris  # install scikit-learn  # noqa: F401
import requests as r  # noqa: F401
import numpy, pandas  # noqa: E401,F401
